# code by jw2pfd
# version 1.0
# Lemma Soft Forums - http://lemmasoft.renai.us/forums/
# Thread discussion - http://lemmasoft.renai.us/forums/viewtopic.php?f=51&t=20704

define allowedch = "abcdefghijklmnopqrstuvwxyz'-"

init python:
    
    import renpy.store as store
    import renpy.exports as renpy

    #used by other classes to inherit the ability to work with rollback
    class Link(store.object):
        pass
    
    #helps handle/manipulate data
    class Wordplay():
        def _listtostr(self, list):
            i = 0
            
            str = ''
            
            while i < len(list) - 1:
                str += list[i] + ' '
                i += 1
                
            str += list[i]            
            str = str.lower()
            
            return str
            
        def _furthersplit(self, str):
            list = str.split(',')            
            i = 0
                
            while i < len(list):
                list[i] = list[i].split()
                i += 1
            
            return list
            
#### Synonyms class ########################################
# is its own class, gets inherited by the Sentence class
# has functions necessary for handling all things related
# to synonyms
    class Synonyms(Wordplay, Link):
        def __init__(self):
            self.clearsynonyms()
            
        #clears the synonym data by setting the dictionary to empty
        def clearsynonyms(self):
            self._synonyms = { }
                
        #addsynonyms(str, str) - adds synonyms to reference during certain function calls 
        #the first argument is a word that we are going to define synonyms for
        #the second argument is a string of word/phrases that are separated by commas
        def addsynonyms(self, word, str):
            list = self._furthersplit(str)
            
            if word not in self._synonyms.keys():
                self._synonyms[word] = list                
            else:
                for item in list:
                    if item not in self._synonyms[word]:
                        self._synonyms[word].append(item)

        def getkey(self, value):
            if value in self._synonyms.keys():
                return value
            
            for key in self._synonyms.keys():
                for item in self._synonyms[key]:
                    if item == value.split():
                        return key
            return None

        def phrases(self, key):
            list = [ ]
            for item in self._synonyms[key]:
                if len(item) > 1:
                    list.append(item)
                    
            return list

        def words(self, key):
            list = [ ]
            for item in self._synonyms[key]:
                if len(item) == 1:
                    list.append(item)
                    
            return list        

        def equal(self, a, b):
            key = None
            
            if a == b:
                return True
            
            if a in self._synonyms.keys():
                key = a
                value = b.split()
            elif b in self._synonyms.keys():
                value = a.split()
                key = b
                
            if key:
                list = self._synonyms[key]
                if value in list:
                    return True
            else:
                keya = self.getkey(a)
                keyb = self.getkey(b)
                
                if keya and keyb:                
                    return keya == keyb            
            
            return False            

#### Sentence Class #############################################
# currently the main class used for user input
#    
    class Sentence(Synonyms, Link):
        def __init__(self, sentence=None):            
            self.clearsynonyms()
            self.clear()
            self.sentence(sentence)
            
        #clears all sentence data, but the synonym data will remain
        def clear(self):            
            self._sentence = None
            self._nopunc = None
            self._singlespace = None
            self._words = [ ]
            self._wordcount = { }
        
        #stores a raw sentence and processes the data to be used later
        #or simply returns the raw sentence if called with no arguments
        def sentence(self, str=None):
            if str:
                self._sentence = str
                self._nopunc = self.removepunctuation()
                self._words = self._nopunc.split()
                self._countwords()
            return self._sentence
            
        #keeps track of each instance of a word
        def _countwords(self):
            for item in self._words:
                if item not in self._wordcount.keys():
                    self._wordcount[item] = 1
                else:
                    self._wordcount[item] += 1
        
        #returns the amount of times a word occurs in the sentence
        #does not care about the word's synonyms
        def count(self, str):
            if str in self._wordcount:
                return self._wordcount[str]            
            return 0
        
        #returns a reconstructed sentence that is lower case and all
        #words are separated by a single space
        def singlespace(self):
            return self._listtostr(self._words)
            
        #returns a sentence with the punctuation replaced by spaces
        def nopunctuation(self):
            return self._nopunc
        
        #removes all characters that aren't in 'allowedch' which
        #is defined at the top of this file
        def removepunctuation(self):            
            str = self.sentence()
            str = str.lower()
            
            for ch in str:
                if ch not in allowedch:
                    str = str.replace(ch, ' ')
                    
            return str
            
        #returns how many words are in the sentence
        def numberofwords(self):
            return len(self._words)
        
        #returns whether or not a sentence contains a word or words
        #function doesn't reference synonyms at all
        def containsword(self, word):
            str = word.replace(' ', '')
            list = str.split(',')
            ret = True
                        
            for item in list:
                ret = ret and self._containsword(item)
            return ret
        
        #internal function to help with containsword
        def _containsword(self, word):
            for item in self._words:
                if item == word:
                    return True
            return False
            
        #returns whether or not a sentence contains a phrase or phrases
        #function doesn't reference synonyms at all
        def containsphrase(self, phrase):
            ret = True
            matrix = self._furthersplit(phrase)
            
            for list in matrix:
                str = self._listtostr(list)
                ret = ret and self._containsphrase(str)
            
            return ret

        #internal function to help with containsphrase
        def _containsphrase(self, phrase):
            return self.singlespace().find(phrase) != -1
        
        
        #returns whether or not there is a complete match based on the
        #argument passed in.  Each argument needs to find an exact or
        #synonym match
        def containsall(self, str):
            return self.contains(str, all=True)        
        
        #Pass in words and/or phrases separated by a comma
        #function returns the number of passed in values that find
        #a match, either exact or synonym.  The return value range is
        #0 to the number of words/phrases you are checking.  If 'all' is
        #set to True, then will return whether or not all match
        def contains(self, str, all=False):
            matrix = self._furthersplit(str)        
            found = 0
            
            for list in matrix:                     
                
                word = self._listtostr(list)        
                key = self.getkey(word)
                
                if key:
                    if self._synonymcheck(key):
                        found += 1                
                else:
                    if len(list) == 1:
                        if self.containsword(word):
                            found += 1
                        else:             
                            if self.containsphrase(word):
                                found += 1
                                        
            if all:
                return found == len(list)
            else:
                return found
        
        #internal function to help check synonym matches
        def _synonymcheck(self, key):
            wlist = self.words(key)
            plist = self.phrases(key)
            
            if len(key.split()) == 1:
                wlist.append(key.split())
            else:
                plist.append(key.split())
                
            for item in wlist:
                if self.containsword(self._listtostr(item)):
                    return True

            for item in plist:
                if self.containsphrase(self._listtostr(item)):
                    return True
                    
            return False

        #returns whether or not the sentence is empty
        def empty(self):
            return self.numberofwords() == 0

        





